@@ -14,6 +14,7 @@
 package config
 
 import (
+	"errors"
 	"fmt"
 	"os"
 	"path/filepath"
@@ -24,7 +25,7 @@ import (
 	"gopkg.in/yaml.v2"
 )
 
-func LoadFile(paths []string) (*Config, error) {
+func LoadFile(paths []string, expandEnvVars bool) (*Config, error) {
 	cfg := &Config{}
 	for _, p := range paths {
 		files, err := filepath.Glob(p)
@@ -42,6 +43,33 @@ func LoadFile(paths []string) (*Config, error) {
 			}
 		}
 	}
+
+	if expandEnvVars {
+		var err error
+		for i, auth := range cfg.Auths {
+			if auth.Username != "" {
+				cfg.Auths[i].Username, err = substituteEnvVariables(auth.Username)
+				if err != nil {
+					return nil, err
+				}
+			}
+			if auth.Password != "" {
+				password, err := substituteEnvVariables(string(auth.Password))
+				if err != nil {
+					return nil, err
+				}
+				cfg.Auths[i].Password.Set(password)
+			}
+			if auth.PrivPassword != "" {
+				privPassword, err := substituteEnvVariables(string(auth.PrivPassword))
+				if err != nil {
+					return nil, err
+				}
+				cfg.Auths[i].PrivPassword.Set(privPassword)
+			}
+		}
+	}
+
 	return cfg, nil
 }
 
@@ -101,7 +129,7 @@ func (c *Module) UnmarshalYAML(unmarshal func(interface{}) error) error {
 }
 
 // ConfigureSNMP sets the various version and auth settings.
-func (c Auth) ConfigureSNMP(g *gosnmp.GoSNMP) {
+func (c Auth) ConfigureSNMP(g *gosnmp.GoSNMP, snmpContext string) {
 	switch c.Version {
 	case 1:
 		g.Version = gosnmp.Version1
@@ -111,7 +139,12 @@ func (c Auth) ConfigureSNMP(g *gosnmp.GoSNMP) {
 		g.Version = gosnmp.Version3
 	}
 	g.Community = string(c.Community)
-	g.ContextName = c.ContextName
+
+	if snmpContext == "" {
+		g.ContextName = c.ContextName
+	} else {
+		g.ContextName = snmpContext
+	}
 
 	// v3 security settings.
 	g.SecurityModel = gosnmp.UserSecurityModel
@@ -213,6 +246,10 @@ type Lookup struct {
 // Secret is a string that must not be revealed on marshaling.
 type Secret string
 
+func (s *Secret) Set(value string) {
+	*s = Secret(value)
+}
+
 // Hack for creating snmp.yml with the secret.
 var (
 	DoNotHideSecrets = false
@@ -317,3 +354,13 @@ func (re *Regexp) UnmarshalYAML(unmarshal func(interface{}) error) error {
 	re.Regexp = regex
 	return nil
 }
+
+func substituteEnvVariables(value string) (string, error) {
+	result := os.Expand(value, func(s string) string {
+		return os.Getenv(s)
+	})
+	if result == "" {
+		return "", errors.New(value + " environment variable not found")
+	}
+	return result, nil
+}
